<?php
/*
Plugin Name: Projekktor Video Tag Extension
Plugin URI: http://www.projekktor.com/cms.php#wordpress
Description: Adds shortcodes to embed HTML5 video with automatic flash-fallback based on the skinnable, free (GPL) Projekktor Media Player. Additionally generates a blogwide playlist with permalinks to the original posts for an optional webTV experience. | <a href="http://www.projekktor.com/cms.php#wordpress" title="Documentation">Online Documentation</a>.
Author: Sascha Kluger
Version: 0.9.3
Author URI: http://www.spinningairwhale.com

Copyright 2010, Sascha Kluger, Spinning Airwhale Media, http://www.spinningairwhale.com
under GNU General Public License
http://www.projekktor.com/license/
*/

add_action('init', 'projekktor_init');
function projekktor_init(){

	$plpath = WP_PLUGIN_URL . '/'.basename((dirname(__FILE__)));

	if (get_option('projekktor_projekktor')!=='false') {		
		wp_enqueue_script('jquery');
		$pSource = $plpath . '/js/projekktor-c.js';
		wp_enqueue_script('projekktor', $pSource , array('jquery'), '0.7.8');
	}
	add_action('admin_menu', 'projekktor_config_page');
	add_action('save_post','projekktor_item_save');
	add_feed('projekktor', 'projekktor_generate_playlist');
}


/* install function - create playlist database */
$projekktor_db_version = "1.0";
register_activation_hook( __FILE__,'projekktor_install');
function projekktor_install() {
	
	global $wpdb;
	global $projekktor_db_version;

	$table_name = $wpdb->prefix . "projekktor_playlist";

	if($wpdb->get_var("show tables like '$table_name'") != $table_name) {
		
		$sql = "CREATE TABLE " . $table_name . " (
		postId mediumint(9) NOT NULL ,
		tagId mediumint(9) NOT NULL,
		created datetime NOT NULL DEFAULT '0000-00-00 00:00:00',
		title tinytext NOT NULL,
		sources text NOT NULL,
		status VARCHAR(55) NOT NULL
		);";
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
		dbDelta($sql);      
		add_option("projekktor_db_version", $projekktor_db_version);
	}
}

function projekktor_get_local_themes() {
	
	$result = array();
	$dir = dirname(__FILE__).'/themes/';
	$d = @dir($dir);
	while(false !== ($entry = $d->read())) {
		if(!preg_match('/projekktor_theme_(.*)\./i', $entry, $matches)) continue;
		$type = (substr(strtolower($entry),-3)=='css') ? 'css' : 'img';
		if ($type=='img') {if (!getimagesize($dir.$entry)) continue;}
		$result[$matches[1]][$type] = $entry;
	}
	$d->close();
	return $result; 	
}


/* fetch available themes from homepage */
function projekktor_get_remote_themes($themeId='') {

	$param = ($themeId!='') ? '?theme='.$themeId : '';
	$html = '<ul>';
	$jsonData = projekktor_fsock_download('themes.projekktor.com', 'index.php'.$param, false);

	if (!$data = json_decode($jsonData)) {
    	echo '<div class="updated fade">'.$jsonData.'</div>';    
        return;    
    }
    
	if ($themeId!='') return $data;
	
	$currentThemeId = get_option('projekktor_themeId');
	foreach($data as $themeIds => $themeData)
	{
		$checked = ($value==$themeId) ? ' checked' : '';
		$style = ($checked!='') ? 'background-color:#FFFFE0;' : '';
		$html .= '<li style="padding: 3px;border: 1px solid #747474; text-align:center; width: 250px;-moz-border-radius:6px;'.$style.'">'; 
		$html .= $themeData->name.' <br/> by ';
		$html .= '<a href="'.$themeData->authorURI.'">'.$themeData->author.'</a>';
		$html .= '<p><img src="'.$themeData->previewImg.'" width="150" height="150"/></p>';
		$html .= '<input type="radio" id="projekktor_themeid" name="projekktor_themeid" value="'.$themeIds.'-'.$themeData->version.'"'.$checked.'>';			

		$html .= '</li>';
	}
	
	$html .= '</ul>';
	return $html;
}

function projekktor_fsock_download($remoteSite, $remoteFile, $localFileLoc=false) {
	
	$fp = fsockopen($remoteSite, 80, $errno, $errstr, 30);
	
	if(!$fp)	
    {	
        return 'Unable to open connection with '.$remoteSite;		
	}
	else {
		$out = "GET /$remoteFile HTTP/1.0\r\n";
		$out .= "Host: $remoteSite\r\n";
		$out .= "Connection: Close\r\n\r\n";
		fwrite($fp, $out);
		$data = '';

		while (!feof($fp)) {
			$data .= fgets($fp, 128);
		}

		//seperate the header and actual content
		$responseData = explode("\r\n\r\n", $data);
	
	
		if ($localFileLoc===false)
		{
			return $responseData[1]; 
		}
		
		if($localFile = fopen($localFileLoc, 'w'))
		{
			if(! fwrite($localFile, $responseData[1]))
			{
				return 'Could not write to local file';
			}
		}
		
		@fclose($fp);
		@fclose($localFile);
		
		return true;
	}
}




function projekktor_get_media_types($selector='video') {
	
	if ($selector=='video') {
		// available video types
		return array(
			'ogg'	=>	'video/ogg',
			'ogv'	=>	'video/ogg',
			'mp4'	=> 	'video/mp4',
			'flv'	=>	'video/flv',   
			'mov'	=>	'video/quicktime',
            'webm'  =>  'video/webm',
			'yt'	=>	'video/youtube'
		);
	}
}

/* stuff this plugin will (more or less) dynamically configure  */
function projekktor_config_options() {
	
	$plpath = WP_PLUGIN_URL . '/'.basename((dirname(__FILE__)));
	$config = array(
		'projekktor' => array(
			'txt' => __('Enable Projekktor (off=use video shortcodes, but disable Projekktor Video Extension):'),
			'val' => 'true',
			'prk' => false
		),
		'enableFlashFallback' => array(
			'txt' => __('Automatically use flash fallback if required and possible:'),
			'val' => 'true',
		),				
		'volume' => array(
			'txt' => __('Initial sound volume (0=muted, 1=max) (optional):'),
			'val' => 0.5
		),
		'logo' => array(
			'txt' => __('Full URL to logo-image - preferably a PNG (optional):'),
			'val' => ''
		),
		'poster' => array(
			'txt' => __('Full URL to a default poster image (optional):'),
			'val' => ''
		),
		'themeId' => array(
			'txt' => __('Player Theme to use:'),
			'val' => 'tll',
			'prk' => false,
			'edt' => false
		),			
		'playerFlashMP4' => array(
			'txt' => __('Path to flash fallback video component'),
			'val' => $plpath."/flash/playerMP4.swf",
			'edt' => false
		),
		'playerFlashMP3' => array(
			'txt' => __('Path to flash fallback audio components'),
			'val' => $plpath."/flash/playerMP3.swf",
			'edt' => false
		)        
		
	);
	return $config;
}


/* apply projekktor config */
add_action('wp_head','projekktor_header');
function projekktor_header() {	
	$myConfigSet = array();
	if (get_option('projekktor_projekktor')==='false') return;
	foreach(projekktor_config_options() as $configKey => $valueSets) {
		if ($valueSets['prk']===false) continue;
		$value = get_option('projekktor_'.$configKey);
		$value = ($value==false) ? $valueSets['val'] : $value;
		$value = ($value+0==0) ? ( ($value=='false'||$value=='true') ? $value : '"'.$value.'"') : $value;
		$myConfigSet[] =  $configKey.':'.$value;
	}
	
	$selectedTheme = explode('-',get_option('projekktor_themeId'));
	$plpath = WP_PLUGIN_URL . '/'.basename((dirname(__FILE__))).'/themes/'.$selectedTheme[0];
	
	echo '<link rel="stylesheet" href="'.$plpath.'/style.css?ver='.$selectedTheme[1].'" type="text/css" media="screen" />';
	echo '<script type="text/javascript">';
	echo 'var PROJEKKTOR = [];';
	echo 'var PROJEKKTOR_CONFIG = {'.join(",",$myConfigSet).'};';
	echo '</script>';
}


/* parse shortcodes */
add_shortcode('video', 'projekktor_shortcodes');
function projekktor_shortcodes( $attributes, $content = null ) {

	// available video types
	$videoTypes = projekktor_get_media_types('video');
	
	// set defaults
	$attributes['poster'] = ($attributes['poster']!= '') ?  ' poster="' . $attributes['poster'] . '" ' : '';
    $attributes['poster'] =( $attributes['poster']== '') ?  ' poster="' . get_option('projekktor_poster') . '" ' : $attributes['poster'];
	$attributes['width'] = ($attributes['width']==0) ? 480 : $attributes['width'];
	$attributes['height'] = ($attributes['height']==0) ? 320 : $attributes['height'];
	$attributes['options'] = ($attributes['options']=='') ? 'controls' : $attributes['options'];
	$attributes['src'] = ($attributes['src']!='') ? 'src="'.$attributes['src'].'" ' : '';
	$attributes['type'] = ($attributes['type']!='') ? ' type="'.$attributes['type'].'" ' : '';	
	$attributes['type'] = ($attributes['src']!='' && $attributes['type']=='') ? ' type="text/json" ' : '';
	
	
	// build video tag
	$output = '<video'.$attributes['type'].$attributes['src'].' class="projekktor" width="'.$attributes['width'].'" height="'.$attributes['height'].'" '.$attributes['poster'].$attributes['options'].'>'."\n";		
	foreach($videoTypes as $videoExtension => $videoType) {		
		if ($attributes['src_'.$videoExtension]=='') continue;
		$output .= '<source src="'.$attributes['src_'.$videoExtension].'" type="'.$videoType.'" />'."\n";				
	} 	
	$output .= '</video>'."\n";
	
	// debugging output:
	// $output .= '<pre>'.htmlentities($output).'</pre>';
	
	return $output;

}


/* projekktor config init */
function projekktor_config_page() {
	if ( function_exists('add_submenu_page') ) {
		add_submenu_page('plugins.php', __('Projekktor Config'), __('Projekktor Config'), 'manage_options', 'projekktor-config', 'projekktor_config');
		// add_plugins_page( __('Projekktor Theme'), __('Projekktor Theme'), 'manage_options', 'projekktor-theme', 'projekktor_theme');
	}
}

/* projekktor theme panel  */
function projekktor_theme() {
	
	echo '<div class="wrap"><h2>'.__('Projekktor Themes').'</h2>';
	
	if ( isset($_POST['submit']) ) {
		
		if ( function_exists('current_user_can') && !current_user_can('manage_options') ) {
			die(__('Cheatin&#8217; uh?'));
		}

		if (isset($_POST['projekktor_themeid']))
		{
			$selectedTheme = explode('-',$_POST['projekktor_themeid']);
			$destDir = dirname(__FILE__).'/themes/'.$selectedTheme[0];
			if (get_option('projekktor_themeId')!=$_POST['projekktor_themeid'] || !file_exists($destDir))
			{				
				if (!file_exists($destDir))
				{
					mkdir($destDir);
				} else {
					$handle=opendir($destDir);
					while (($file = readdir($handle))!==false) {
						@unlink($dir.'/'.$file);
					}
					closedir($handle);				
				}
				
				$themeFiles = projekktor_get_remote_themes($selectedTheme[0]);	
				foreach($themeFiles->$selectedTheme[0]->files as $void => $themeFile)
				{
					if (file_exists($destDir.'/'.$themeFile)) continue;
					projekktor_fsock_download('themes.projekktor.com', $selectedTheme[0].'/'.$themeFile, $destDir.'/'.$themeFile);
				}
				echo '<div class="updated fade">Themefiles updated. Saved!</div>';
			} else {
				echo '<div class="updated fade">Saved!</div>';
			}
			update_option('projekktor_themeId', $_POST['projekktor_themeid']);
			
		}
	}
	

	echo '<div class="narrow"><form action="" method="post" id="projekktor-conf" style="margin: auto; ">';
	echo projekktor_get_remote_themes();
	echo '<p class="submit"><input type="submit" name="submit" value="'.__('Download and set selected theme &raquo;').'" /></p>';
	echo '</form></div> <!-- close narrow --></div><!-- close wrap -->';
	
}


/* projekktor config panel itself */
function projekktor_config() {
	
	if (isset($_GET['theme'])) {
		projekktor_theme();
		return;
	}
	
	
	if ( isset($_POST['submit']) ) {
		
		if ( function_exists('current_user_can') && !current_user_can('manage_options') ) {
			die(__('Cheatin&#8217; uh?'));
		}
		
		foreach(projekktor_config_options() as $configKey => $valueSets) {
			if (!isset($_POST['projekktor_'.$configKey])) continue;
			if ($valueSets['edt']===false) continue;
			update_option('projekktor_'.$configKey, $_POST['projekktor_'.$configKey]);
		}
		echo '<div class="updated fade">Saved!</div>';
	}
	
	$plpath = WP_PLUGIN_URL . '/'.basename((dirname(__FILE__)));
	$fieldsHTML = '';
	
	foreach(projekktor_config_options() as $configKey => $valueSets) {
		
		if ($configKey=='theme') continue;
		if ($valueSets['edt']===false) continue;
		
		$value = get_option('projekktor_'.$configKey);
		$value = ($value===false) ? $valueSets['val'] : $value;

		$size = (is_string($valueSets['val'])) ? 120 : 12;

		$fieldsHTML .= '<p>';
		$fieldsHTML .= '<label for="projekktor_'.$configKey.'">'.$valueSets['txt'].'</label> ';		

		if ($valueSets['val']==='true' || $valueSets['val']==='false') {
			$checked = ($value=='false') ? ' checked' : '';
			$fieldsHTML .=  '<input type="radio" id="projekktor_'.$configKey.'" name="projekktor_'.$configKey.'" value="false"'.$checked.'> OFF / ';							
			$checked = ($value=='true') ? ' checked' : '';
			$fieldsHTML .=  '<input type="radio" id="projekktor_'.$configKey.'" name="projekktor_'.$configKey.'" value="true"'.$checked.'> ON ';			
		}else {
			$fieldsHTML .= '<br/><input id="projekktor_'.$configKey.'" name="projekktor_'.$configKey.'" type="text" size="'.$size.'" maxlength="'.(($size<50) ? '12' : $size*100).'" value="'.$value.'" />';
		}

		$fieldsHTML .= '</p>';
	}
	
	echo '<div class="wrap"><h2>'.__('Projekktor Config').'</h2>';
	echo '<div class="narrow"><form action="" method="post" id="projekktor-conf" style="margin: auto; ">';
	echo $fieldsHTML;
	echo '<p class="submit"><input type="submit" name="submit" value="'.__('Update options &raquo;').'" /></p>';
	echo '<b><a href="?page=projekktor-config&theme=true">Download and set Projekktor theme</a></b>';
	echo '</form></div> <!-- close narrow --></div><!-- close wrap -->';
	
	
}


/* save videoitems in posts to playlist */
function projekktor_item_save($postId) {
	
	global $wpdb;

	$videoTypes = projekktor_get_media_types('video');	
	$table_name = $wpdb->prefix . "projekktor_playlist";
	$post = get_post($postId);

	if ($post->post_type!='post' && $post->post_type!='revision') return;

	// delete all old playlist entries:
	$sql = "DELETE FROM " . $table_name . " WHERE postId=".$postId.";";
	$results = $wpdb->query( $sql );	
	
	// parse video tags	
	$pattern = '(.?)\[(video)\b(.*?)(?:(\/))?\](?:(.+?)\[\/\2\])?(.?)';
	preg_match_all('/'.$pattern.'/s', $post->post_content, $tagMatches);
	
	if (!$tagMatches[3]) return;
	
	// gather video entities
	$videoObjects = array();
	foreach($tagMatches[3] as $key => $tagEntitiy) {	
		$attribs = shortcode_parse_atts($tagEntitiy);
		if (count($attribs)==0) continue;
		foreach($attribs as $attribName => $attribValue) {
			if (substr($attribName,0,3)=='src') {
				$videoObjects[$key]['sources'][] = array(
					'src'	=> 	$attribValue,
					'type'	=> 	$videoTypes[substr($attribName,4)]
				);
				continue;
			}
			$videoObjects[$key][$attribName] = $attribValue;
		}
	}

	// INSERT entities to DB
	foreach($videoObjects as $key => $videoData) {
		$title = ($videoData['title']!='') ? $videoData['title'] : $post->post_title;
		$sql = "INSERT INTO " . $table_name . " (created, title, postId, tagId, sources, status) " .
		"VALUES ('" . $post->post_date . "','" . $wpdb->escape($title) . "',".$postId.", '".$key."' ,'" . serialize($videoData['sources']) . "', '".$post->post_status."')";
		$results = $wpdb->query( $sql );	
	}

}


/* generate projekktor playlist (webTV mode) */
function projekktor_generate_playlist($content) {
	
	global $wpdb;
	$table_name = $wpdb->prefix . "projekktor_playlist";
	$sql = "SELECT created, postId, title,sources FROM " . $table_name . " WHERE status='publish' ORDER BY created DESC;";
	$results = $wpdb->get_results( $sql );
	
	$playlist = array();
	$count = 0;
	foreach($results as $resultKey => $rowObject) {
		$playlist["$count "] = array(
			'title'		=> 	$rowObject->title,
			'created'	=> 	strtotime($rowObject->created),
			'link'		=>	get_permalink( $rowObject->postId ),
			'file'		=>	unserialize($rowObject->sources)
		);
		$count++;
	}
	
	header("Content-Type: application/json");
	echo json_encode($playlist);
	
}




/*
*    ____       
*   /# /_\_     
*  |  |/o\o\   ,-----------------------------,      
*  |  \\_/_/   |  sigh - what a stupid mess. | 
* / |_   |     |-----------------------------/  
*|  ||\_ ~|  _/ 
*|  ||| \/     
*|  |||_       
* \//  |       
*  ||  |       
*  ||_  \      
*  \_|  o|     
*  /\___/      
* /  ||||__    
*   (___)_)
*/
